use crate::read;

pub fn run() {
    let grid: Vec<Vec<u32>> = read("08")
        .lines()
        .map(|l| {
            l.chars()
                .map(|c| c.to_digit(10).unwrap())
                .collect::<Vec<_>>()
        })
        .collect::<Vec<_>>();

    let width = grid.first().unwrap().len();
    let mut count = 0;
    let mut max_score = 0;

    for (row, row_vec) in grid.iter().enumerate().skip(1) {
        for (col, height) in row_vec.iter().enumerate().skip(1) {
            let mut sides_visible = 4;
            let mut score = 1;

            score *= if let Some(sc_score) = (0..col)
                .rev()
                .map(|cc| (col - cc, row_vec[cc]))
                .find(|(_sc_score, other_height)| other_height >= height)
            {
                sides_visible -= 1;
                sc_score.0
            } else {
                col
            };

            score *= if let Some(sc_score) = (col + 1..width)
                .map(|cc| (cc - col, row_vec[cc]))
                .find(|(_sc_score, other_height)| other_height >= height)
            {
                sides_visible -= 1;
                sc_score.0
            } else {
                width - col - 1
            };

            score *= if let Some(sc_score) = (0..row)
                .rev()
                .map(|rr| (row - rr, grid[rr][col]))
                .find(|(_sc_score, other_height)| other_height >= height)
            {
                sides_visible -= 1;
                sc_score.0
            } else {
                row
            };

            score *= if let Some(sc_score) = grid
                .iter()
                .enumerate()
                .skip(row + 1)
                .map(|(rr, l)| (rr - row, l[col]))
                .find(|(_sc_score, other_height)| other_height >= height)
            {
                sides_visible -= 1;
                sc_score.0
            } else {
                grid.len() - row - 1
            };

            max_score = max_score.max(score);

            if sides_visible > 0 {
                count += 1;
            }
        }
    }

    #[cfg(feature = "part1")]
    {
        println!("Day 8, Part 01: {}", count);
    }

    #[cfg(feature = "part2")]
    {
        println!("Day 8, Part 02: {}", max_score);
    }
}
